
use [NetPhoneTest]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_GetHash]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[sp_GetHash]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_GetPhoneInfo]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[sp_GetPhoneInfo]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_SetPasswordStatus]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[sp_SetPasswordStatus]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_StoreTestResults]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[sp_StoreTestResults]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[ASN2Hash]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
drop table [dbo].[ASN2Hash]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[Phones]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
drop table [dbo].[Phones]
GO

CREATE TABLE [dbo].[ASN2Hash] (
	[ASN] [varchar] (12) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
	[PwdHash] [varchar] (255) COLLATE SQL_Latin1_General_CP1_CI_AS NULL ,
	[Used] [bit] NULL ,
	[Pwd] [char] (16) COLLATE SQL_Latin1_General_CP1_CI_AS NULL 
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[Phones] (
	[MAC] [varchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
	[MAC2] [varchar] (50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
	[DateBuild] [datetime] NOT NULL ,
	[DateCheck] [datetime] NULL ,
	[Keys] [int] NOT NULL ,
	[Leds] [int] NOT NULL ,
	[Audio] [int] NOT NULL ,
	[Display] [int] NOT NULL ,
	[IsPwdSet] [int] NOT NULL ,
	[IsMAC1OK] [int] NOT NULL ,
	[IsMAC2OK] [int] NOT NULL ,
	[IsSNOK] [int] NOT NULL ,
	[IsASNOK] [int] NOT NULL ,
	[ASN] [varchar] (12) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
	[SN] [varchar] (12) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[Phones] WITH NOCHECK ADD 
	CONSTRAINT [PK_Phones] PRIMARY KEY  CLUSTERED 
	(
		[MAC]
	)  ON [PRIMARY] 
GO

ALTER TABLE [dbo].[ASN2Hash] ADD 
	CONSTRAINT [DF_ASN2Hash_Used] DEFAULT (0) FOR [Used]
GO

ALTER TABLE [dbo].[Phones] ADD 
	CONSTRAINT [DF_Phones_DateBuild] DEFAULT (getdate()) FOR [DateBuild],
	CONSTRAINT [DF_Phones_Keys] DEFAULT (0) FOR [Keys],
	CONSTRAINT [DF_Phones_Leds] DEFAULT (0) FOR [Leds],
	CONSTRAINT [DF_Phones_Speakers] DEFAULT (0) FOR [Audio],
	CONSTRAINT [DF_Phones_IsPwdSet] DEFAULT (0) FOR [IsPwdSet],
	CONSTRAINT [DF_Phones_IsMAC2OK1] DEFAULT (0) FOR [IsMAC1OK],
	CONSTRAINT [DF_Phones_IsMAC2OK] DEFAULT (0) FOR [IsMAC2OK],
	CONSTRAINT [DF_Phones_IsSNOK] DEFAULT (0) FOR [IsSNOK],
	CONSTRAINT [DF_Phones_IsSNOK1] DEFAULT (0) FOR [IsASNOK]
GO

SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

CREATE PROCEDURE dbo.sp_GetHash
@nError int output ,
@strPwdHash nvarchar(255) output,
@strASN nvarchar(12) 
 AS

	IF @strASN = '-' BEGIN
		SET @strPwdHash = '$1$O9tVzmaZ$lEUR.xPO7p2cUFvKFzzRF0'  
		SET @nError = 0
	END ELSE BEGIN
		SELECT 
			@strPwdHash = PwdHash
			FROM 
				ASN2Hash
			WHERE
				ASN = @strASN
	
		IF @@ROWCOUNT = 0 BEGIN
			SET @nError = -2
		END ELSE BEGIN
			SET @nError = 0
		END
	END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS OFF 
GO

CREATE PROCEDURE dbo.sp_GetPhoneInfo
@nError int output ,
@strMAC2 as nvarchar(12) output,
@strASN nvarchar(12) output,
@strSN nvarchar(12) output,
@strMAC as nvarchar(12)
 AS
DECLARE 
	@dateCheck as datetime
SELECT 
	@strMAC = P.MAC2,
	@strASN = P.ASN,
	@strSN = P.SN,
	@dateCheck =  DateCheck
	FROM 
		Phones P 
	WHERE
		P.MAC = @strMAC
	
IF @@ROWCOUNT =  0 BEGIN
	SET @nError = -1
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

CREATE PROCEDURE dbo.sp_SetPasswordStatus
@nError int output ,
@strMAC as nvarchar(12),
@nPwdSet  int 
 AS

UPDATE Phones
	SET IsPwdSet =  @nPwdSet
	WHERE
		MAC = @strMAC
	IF @@ROWCOUNT = 0 BEGIN
		SET @nError = -1
	END ELSE BEGIN
		SET @nError = 0
	END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS OFF 
GO

CREATE PROCEDURE dbo.sp_StoreTestResults
@nError int output ,
@strPwdHash nvarchar(255) output,
@strMAC as nvarchar(12),
@strMAC2 as nvarchar(12),
@strASN nvarchar(12),
@strSN nvarchar(12),
@nKeys int ,
@nLeds int ,
@nAudio int ,
@nDisplay int, 
@nIsMAC1OK int, 
@nIsMAC2OK int, 
@nIsSNOK int,
@nIsASNOK int
 AS

BEGIN TRANSACTION
DELETE 
	FROM Phones 
	WHERE 
		MAC = @strMAC

INSERT INTO Phones(MAC, MAC2,  ASN, SN, Keys, Leds, Audio, Display, DateCheck, IsMAC1OK,  IsMAC2OK, IsSNOK,  IsASNOK, IsPwdSet)
	VALUES(
		@strMAC, @strMAC2, @strASN, @strSN, 
		@nKeys, @nLeds, @nAudio, @nDisplay, 
		getdate(), @nIsMAC1OK, @nIsMAC2OK, @nIsSNOK,  @nIsASNOK,0
	)

IF @@ROWCOUNT = 0 BEGIN
	SET @nError = -1
END ELSE BEGIN
	IF @strASN = '-' BEGIN
		SET @strPwdHash = '$1$O9tVzmaZ$lEUR.xPO7p2cUFvKFzzRF0'  
		SET @nError = 0
	END ELSE BEGIN
	SELECT 
		@strPwdHash = PwdHash
		FROM 
			ASN2Hash
		WHERE
			ASN = @strASN

	IF @@ROWCOUNT = 0 AND  @strASN <> '-' BEGIN
		SET @nError = -2
	END ELSE BEGIN
		SET @nError = 0
	END
	END
END	

COMMIT TRANSACTION
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

