#include "stdafx.h"
#include "PXATester.h"
#include "Phone.h"
#include "Helpers.h"
#include "PhoneDlg.h"


#import "C:\Program Files\Common Files\System\ADO\msado15.dll" rename( "EOF", "adoEOF" )

#define TEST_CONN_STRING _T("Provider=sqloledb;Data Source=%s;Initial Catalog=%s;User Id=%s;Password=%s")


CPhone::CPhone()
{
	memset(m_keys, 0, sizeof(Key)*PHONE_KEYS);
	memset(m_leds, 0, sizeof(Led)*PHONE_LEDS);
	memset(&m_audio, 0, sizeof(Audio));
	memset(&m_display, 0, sizeof(Display));
	memset(m_mac, 0, sizeof(BYTE)*8);
	m_nTest = tLast;
	m_bPhoneOK = false;	
}

void CPhone::SetDlg(CPhoneDlg *pDlg)
{
	m_pDlg = pDlg;
}

bool CPhone::OnConnected()
{
	CString strAddr;
	UINT nPort;

	m_strMAC.Empty();
	m_strMAC2.Empty();
	m_strSN.Empty();
	m_strASN.Empty();
	
	memset(m_keys, 0, sizeof(Key)*PHONE_KEYS);
	memset(m_leds, 0, sizeof(Led)*PHONE_LEDS);
	memset(&m_audio, 0, sizeof(Audio));
	memset(&m_display, 0, sizeof(Display));
	memset(m_mac, 0, sizeof(BYTE)*8);
	m_pDlg->SetButtonsStatus(true);
	m_nTest = tKbd;
	m_bPhoneOK = false;	

	GetPeerName(strAddr, nPort);
	
	if (!GetRemoteMACAddress(strAddr, m_mac))
	{	// Could not find mac address of phone
		StoreTestResults();
		AfxMessageBox(IDS_ERR_COULD_NOT_FIND_MAC, MB_OK | MB_ICONSTOP); 
		return false;
	}

	m_strMAC.Format( _T("%02X%02X%02X%02X%02X%02X"), m_mac[0], m_mac[1], m_mac[2], m_mac[3], m_mac[4], m_mac[5] );
	
	if ( 0 != m_strMAC.CompareNoCase(m_pDlg->GetMAC1()))
	{
		StoreTestResults();
		CString str;
		str.Format(IDS_ERR_MAC_NOT_MATCH,
			m_strMAC, 
			m_pDlg->GetMAC1());
		AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
		return false;
	}

	int nErr = GetDBPhoneInfo();
	if (0 == nErr)
	{	// Already tested
		if (0 != m_strSN.CompareNoCase(m_pDlg->GetSNPhone()))
		{
			StoreTestResults();
			CString str;
			str.Format(IDS_ERR_SNPHONE_NOT_MATCH,
				m_strSN, 
				m_pDlg->GetSNPhone());
			AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
			return false;
		}
	
		if (0 != m_strASN.CompareNoCase(m_pDlg->GetSNEnvelope()))
		{
			CString str;
			str.Format(IDS_ERR_SNEnvelope_NOT_MATCH,
				m_strASN, 
				m_pDlg->GetSNEnvelope());
			if (IDYES == AfxMessageBox(str, MB_YESNO  | MB_ICONQUESTION))
			{
				m_strASN = m_pDlg->GetSNEnvelope();
			}
			else
			{
				StoreTestResults();
				return false;
			}
		} else
		{
			AfxMessageBox(IDS_ERR_ALREADY_TESTED, MB_OK | MB_ICONINFORMATION); 
		}
	} else if( nErr == -1)
	{
		m_strMAC2= m_pDlg->GetMAC2();
		m_strSN = m_pDlg->GetSNPhone();
		m_strASN = m_pDlg->GetSNEnvelope();
	} else
	{
		return false;
	}

	nErr = GetDBPwdHash();
	if ( nErr == -2 )
	{
		m_strASN.Empty();
		StoreTestResults();
		CString str;
			str.Format(IDS_ERR_DB_HASH_NOT_FOUND, m_pDlg->GetSNEnvelope());
			AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
		return false;
	}
	CString str;
	str.Format(IDS_PHONE_INFO, (LPCTSTR)m_strMAC, (LPCTSTR)m_pDlg->GetSNPhone(), (LPCTSTR)m_pDlg->GetSNEnvelope());
	m_pDlg->SetTitleText(str);
	Send(_T("START"));
	StartChecker(); 
#ifdef NO_DB
	StartTest();
#endif
	return true;
}

bool CPhone::TestsOK() const
{
	int i;
	for (i=0; i < PHONE_KEYS; i++)
		if (!m_keys[i].nDown || m_keys[i].nDown != m_keys[i].nUp)
			return false;

	for (i=0; i < PHONE_LEDS; i++)
		if (!m_leds[i].bGreen || !m_leds[i].bRed || m_leds[i].bOff)
			return false;

	if (!m_audio.bHandsetMic || !m_audio.bHandsetSpeaker || !m_audio.bPhoneMic || !m_audio.bPhoneSpeaker)	
		return false;

	if (!m_display.bEmpty || !m_display.bFull || !m_display.bMesh)	
		return false;
	return true;
}


bool CPhone::StartTest()
{
	return NextTest();
}


bool CPhone::NextTest()
{
	switch(m_nTest)
	{
	case tKbd: 
		TestKbd(); break;
	case tLED1red:
	case tLED1green:
	case tLED1off:
	case tLED2red:
	case tLED2green:
	case tLED2off:
		TestLED(); break;
	case tDisplayFull:
	case tDisplayMesh:
	case tDisplayEmpty:
		TestDisplay(); break;
	case tHandsetSpeaker:
	case tHandsetMic:
	case tPhoneSpeaker:
	case tPhoneMic:
		TestAudio(); break;
	default:
		return false;
	}
	return true;
};

bool CPhone::TestKbd()
{
	m_pDlg->TestKbd();
	Send(_T("KBD1"));
	return true;
}

bool CPhone::TestLED()
{
	CString strCmd;
	switch(m_nTest)
	{
	case tLED1red:		strCmd = _T("LED1R"); break; 
	case tLED1green:	strCmd = _T("LED1G"); break; 
	case tLED1off:		strCmd = _T("LED1O"); break; 
	case tLED2red:		strCmd = _T("LED2R"); break; 
	case tLED2green:	strCmd = _T("LED2G"); break; 
	case tLED2off:		strCmd = _T("LED2O"); break; 
	}
	Send(strCmd);
	m_pDlg->TestLED(m_nTest);
	return true;
}

bool CPhone::TestDisplay()
{
	CString strCmd;
	switch(m_nTest)
	{
	case tDisplayFull:	strCmd = _T("DIS0"); break; 
	case tDisplayMesh:	strCmd = _T("DIS1"); break; 
	case tDisplayEmpty:	strCmd = _T("DIS2"); break; 
	}
	Send(strCmd);
	m_pDlg->TestDisplay(m_nTest);
	return true;
}
bool CPhone::TestAudio()
{
	CString strCmd;
	switch(m_nTest)
	{
	case tHandsetSpeaker:	strCmd = _T("AUDHS"); break; 
	case tHandsetMic:		strCmd = _T("AUDHM"); break; 
	case tPhoneSpeaker:		strCmd = _T("AUDPS"); break; 
	case tPhoneMic:			strCmd = _T("AUDPM"); break; 
	}
	Send(strCmd);
	m_pDlg->TestAudio(m_nTest);
	return true;
}

bool CPhone::OnTestFinish(bool bOK)
{
	switch(m_nTest)
	{
	case tKbd:		
		Send(_T("KBD0"));
		m_pDlg->SetButtonsStatus(true);
		break;
		
	case tLED1red:		m_leds[0].bRed = bOK; break;
	case tLED1green:	m_leds[0].bGreen = bOK; break;
	case tLED2red:		m_leds[1].bRed = bOK; break;
	case tLED2green:	m_leds[1].bGreen = bOK; break;

	case tDisplayFull:	m_display.bFull = bOK; break;
	case tDisplayMesh:	m_display.bMesh = bOK; break;
	case tDisplayEmpty:	m_display.bEmpty = bOK; break;

	case tHandsetSpeaker:	m_audio.bHandsetSpeaker = bOK; break;
	case tHandsetMic:		m_audio.bHandsetMic = bOK; break;
	case tPhoneSpeaker:		m_audio.bPhoneSpeaker = bOK; break;
	case tPhoneMic:		m_audio.bPhoneMic = bOK; break;
	}
	m_nTest=(eTest)(m_nTest+1);
	if (tLast == m_nTest)
	{	// Last test
		int nErr = StoreTestResults();
		switch (nErr)
		{
		case -1: // Not inserted
			{
				AfxMessageBox(IDS_ERR_DB_CANNOT_INSERT, MB_OK | MB_ICONSTOP);
				m_nTest = tLast;
				m_pDlg->TestFinished();
				m_pDlg->DisconnectPhone(false);
				break;
			}
		case -2: // Not inserted
			{
				AfxMessageBox(IDS_ERR_DB_CANNOT_INSERT, MB_OK | MB_ICONSTOP);
				m_nTest = tLast;
				m_pDlg->TestFinished();
				m_pDlg->DisconnectPhone(false);
				break;
			}
		case 0:
			m_pDlg->TestFinished();
			if (!TestsOK())
			{
				m_pDlg->DisconnectPhone(false);
			} else
			{
				CString strCmd(_T("END"));
				strCmd += TestsOK() ? (_T("1") + m_strPwdHash) : _T("0");
				Send(strCmd);
			}
		}
	} else
	{
		NextTest();
	}
	return true;
}

bool CPhone::OnDisconnected()
{
	if (!IsTestCompleted())
	{ // Disconnected
		StoreTestResults();
		AfxMessageBox(IDS_ERR_PHONE_DISCONNECTED, MB_OK | MB_ICONSTOP); 
		m_nTest = tLast;
	}
	m_pDlg->EndDialog(0);
	Close();
	return true;
}


bool CPhone::OnData(LPCTSTR lpData)
{
	// Read command (terminated with \n);
	CString strCommand = lpData;
	if (_T("MAC2") == strCommand.Left(4))
	{
		m_strMAC2 = strCommand.Mid(4);
		m_strMAC2.Replace(_T("-"), _T(""));
		if (0 != m_pDlg->GetMAC2().CompareNoCase(m_strMAC2))
		{
			CString str;
			str.Format(IDS_ERR_MAC2_NOT_MATCH,
				m_strMAC2, 
				m_pDlg->GetMAC2());
			
			SuspendChecker(); 
			AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
			StoreTestResults();
			ResumeChecker(); 
			m_nTest = tLast;
			m_pDlg->TestFinished();
			m_pDlg->DisconnectPhone(false);
			return true;
		}
		StartTest();
	} 
	else if (_T("END") == strCommand.Left(3))
	{
		CString strCode = strCommand.Mid(3,1);
		if (strCode == "0")
		{
			int nErr = SetPasswordStatus(true);
			m_bPhoneOK = strCode=="0" && TestsOK();
		}
		else if (strCode == "1")
		{
			AfxMessageBox(IDS_ERR_CAN_NOT_OPEN_SHADOW, MB_OK | MB_ICONSTOP); 
			m_bPhoneOK = false;
		}
		else if (strCode == "2")
		{
			AfxMessageBox(IDS_ERR_ROOT_IN_SHADOW_NOT_FOUND, MB_OK | MB_ICONSTOP); 
			m_bPhoneOK = false;
		}
		m_pDlg->DisconnectPhone(m_bPhoneOK);
	} 
	else if (tKbd == m_nTest)
	{
		if (_T("KBD") == strCommand.Left(3))
		{
			CString strCode = strCommand.Mid(4,4);
			unsigned long nCode = _tcstoul( strCode, NULL, 16 );
			switch(strCommand[3])
			{
			case _T('U'):
				MarkButtonAsChecked(nCode, true);
				break;
			case _T('D'):
				MarkButtonAsChecked(nCode, false);
				break;
			default:
				ASSERT(NULL);
				break;
			}
			// Test whether all buttons are tested
			int i;
			bool bOK = true;

			for (i=0; i < PHONE_KEYS; i++)
			{
				if (m_keys[i].nUp==0 || m_keys[i].nUp != m_keys[i].nDown)
				{
					bOK = false;
					break;
				}
			}
			if (bOK)
			{
				OnTestFinish(true);
			}

		}
	}
	return true;
}

void CPhone::MarkButtonAsChecked(int nButton, bool bUp)
{
	int nCode = PhoneKey2Ordinal(nButton);
	if (nCode>=0)
	{
		if (bUp)
			m_keys[nCode].nUp++;
		else
			m_keys[nCode].nDown++;
		
		m_pDlg->SetButtonStatus(nCode, m_keys[nCode].nDown == m_keys[nCode].nUp);

		TRACE(_T("%d-%s"), nCode, bUp ? _T("Up"): _T("Down"));	
	}
}


int CPhone::GetDBPhoneInfo()
{
#ifdef NO_DB
	m_strMAC2 = _T("1234567");
	m_strASN = _T("1234567");
	m_strSN = _T("1234567");
	return 0;
#endif

	int nError=0;
	m_bAlreadyTested = false;
	
	ADODB::_CommandPtr		ptrCommand;
	ADODB::_ConnectionPtr	ptrConnection;
	ADODB::_RecordsetPtr	ptrRecordSet;


	HRESULT hr = S_OK;
	try
	{
		_bstr_t strCnn( (LPCTSTR)GetConnectionString() );
	    hr = ptrConnection.CreateInstance(__uuidof( ADODB:: Connection ));

		ptrConnection->Open(strCnn, "", "", ADODB::adOpenUnspecified);
	
 		hr = ptrCommand.CreateInstance(__uuidof( ADODB:: Command ));
		
	// Tell the Command to use our existing Connection.
		ptrCommand->ActiveConnection = ptrConnection;

		CString strProcedureName;
		strProcedureName = _T("sp_GetPhoneInfo");

		ptrCommand->CommandText = (LPCTSTR)strProcedureName;
		ptrCommand->CommandType = ADODB::adCmdStoredProc; 
//		SetConnectionEvents(ptrConnection);

		
	// Add stored procedure params
		ptrCommand->Parameters->Append(
			ptrCommand->CreateParameter( "@RETURN_VALUE",
                          ADODB::adInteger, 
                          ADODB::adParamReturnValue, 
                          sizeof(int),
 						  _variant_t("0")) );

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nError",
                          ADODB::adInteger, 
                          ADODB::adParamOutput, 0,_variant_t("0")));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter("@strMAC2",
                          ADODB::adVarWChar, 
                          ADODB::adParamOutput, 12,_variant_t("")));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter("@strASN",
                          ADODB::adVarWChar, 
                          ADODB::adParamOutput, 12,_variant_t("")));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strSN",
                          ADODB::adVarWChar, 
                          ADODB::adParamOutput, 12,_variant_t("")));
	
		_variant_t vMAC = (LPCTSTR)m_pDlg->GetMAC1();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strMAC",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vMAC));

		ptrCommand->Execute(NULL,NULL,ADODB::adCmdStoredProc );

		
		_variant_t propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 1 ) ]->Value;
		nError = propValue.intVal;
		propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 2 ) ]->Value;
		m_strMAC2 = (LPCTSTR)(propValue.bstrVal);
		propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 3 ) ]->Value;
		m_strASN = (LPCTSTR)(propValue.bstrVal);
		propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 4 ) ]->Value;
		m_strSN = (LPCTSTR)(propValue.bstrVal);
	}
	catch(_com_error &e)
	{
	// Notify the user of errors if any.
    // Pass a connection pointer accessed from the Connection.
//		PrintProviderError(m_ptrConnection);
      _bstr_t bstrSource(e.Source());
      _bstr_t bstrDescription(e.Description());

//      PrintProviderError(pConnection);

		CString str;
		str.Format(IDS_ERR_DB,
			(LPCTSTR)bstrSource,(LPCTSTR)bstrDescription);
		AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
	  nError = 1;
	}

	m_bAlreadyTested = nError == 0;

	return nError;
}

int CPhone::GetDBPwdHash()
{
#ifdef NO_DB
	return 0;
#endif

	int nError=0;
	m_bAlreadyTested = false;
	
	ADODB::_CommandPtr		ptrCommand;
	ADODB::_ConnectionPtr	ptrConnection;
	ADODB::_RecordsetPtr	ptrRecordSet;


	HRESULT hr = S_OK;
	try
	{
		_bstr_t strCnn( (LPCTSTR)GetConnectionString() );
	    hr = ptrConnection.CreateInstance(__uuidof( ADODB:: Connection ));

		ptrConnection->Open(strCnn, "", "", ADODB::adOpenUnspecified);
	
 		hr = ptrCommand.CreateInstance(__uuidof( ADODB:: Command ));
		
	// Tell the Command to use our existing Connection.
		ptrCommand->ActiveConnection = ptrConnection;

		CString strProcedureName;
		strProcedureName = _T("sp_GetHash");

		ptrCommand->CommandText = (LPCTSTR)strProcedureName;
		ptrCommand->CommandType = ADODB::adCmdStoredProc; 
//		SetConnectionEvents(ptrConnection);

		
	// Add stored procedure params
		ptrCommand->Parameters->Append(
			ptrCommand->CreateParameter( "@RETURN_VALUE",
                          ADODB::adInteger, 
                          ADODB::adParamReturnValue, 
                          sizeof(int),
 						  _variant_t("0")) );

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nError",
                          ADODB::adInteger, 
                          ADODB::adParamOutput, 0,_variant_t("0")));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strPwdHash",
                          ADODB::adVarWChar, 
                          ADODB::adParamOutput, 255,_variant_t("")));

		_variant_t vSN = (LPCTSTR)m_pDlg->GetSNEnvelope();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strASN",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vSN));

	
		ptrCommand->Execute(NULL,NULL,ADODB::adCmdStoredProc );

		
		_variant_t propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 1 ) ]->Value;
		nError = propValue.intVal;
		propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 2 ) ]->Value;
		m_strPwdHash = (LPCTSTR)(propValue.bstrVal);
	}
	catch(_com_error &e)
	{
	// Notify the user of errors if any.
    // Pass a connection pointer accessed from the Connection.
//		PrintProviderError(m_ptrConnection);
      _bstr_t bstrSource(e.Source());
      _bstr_t bstrDescription(e.Description());

//      PrintProviderError(pConnection);

		CString str;
		str.Format(IDS_ERR_DB,
			(LPCTSTR)bstrSource,(LPCTSTR)bstrDescription);
		AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
	  nError = 1;
	}

	return nError;
}

int CPhone::StoreTestResults()
{
#ifdef NO_DB
	return 0;
#endif
	int nError=0;
	int nKeys; 
	int nLeds;
	int nAudio;
	int nDisplay;

	ADODB::_CommandPtr		ptrCommand;
	ADODB::_ConnectionPtr	ptrConnection;
	ADODB::_RecordsetPtr	ptrRecordSet;


	int i;
	nKeys = 0;
	for (i=0; i < PHONE_KEYS; i++)
	{
		nKeys <<= 1;
		nKeys |= (!m_keys[i].nDown || m_keys[i].nDown != m_keys[i].nUp) ? 0 : 1;
	}
	nLeds = 0;
	for (i=0; i < PHONE_LEDS; i++)
	{
		nLeds <<= 1; nLeds |= m_leds[i].bGreen ? 1 : 0; 
		nLeds <<= 1; nLeds |= m_leds[i].bRed ? 1 : 0;
		nLeds <<= 1; nLeds |= m_leds[i].bOff ? 1 : 0;
	}
	nAudio = 0;
	nAudio |= m_audio.bHandsetMic ? 1 : 0;
	nAudio |= m_audio.bHandsetSpeaker ? 2 : 0;
	nAudio |= m_audio.bPhoneMic ? 4 : 0;
	nAudio |= m_audio.bPhoneSpeaker ? 8 : 0;
	
	nDisplay = 0;
	nDisplay |= m_display.bEmpty ? 1 : 0 ;
	nDisplay |= m_display.bFull ? 2 : 0 ;
	nDisplay |= m_display.bMesh ? 4 : 0 ;

	HRESULT hr = S_OK;
	try
	{
		_bstr_t strCnn( (LPCTSTR)GetConnectionString() );
	    hr = ptrConnection.CreateInstance(__uuidof( ADODB:: Connection ));

		ptrConnection->Open(strCnn, "", "", ADODB::adOpenUnspecified);
	
 		hr = ptrCommand.CreateInstance(__uuidof( ADODB:: Command ));
		
	// Tell the Command to use our existing Connection.
		ptrCommand->ActiveConnection = ptrConnection;

		CString strProcedureName;
		strProcedureName = _T("sp_StoreTestResults");

		ptrCommand->CommandText = (LPCTSTR)strProcedureName;
		ptrCommand->CommandType = ADODB::adCmdStoredProc; 
//		SetConnectionEvents(ptrConnection);

		
	// Add stored procedure params
		ptrCommand->Parameters->Append(
			ptrCommand->CreateParameter( "@RETURN_VALUE",
                          ADODB::adInteger, 
                          ADODB::adParamReturnValue, 
                          sizeof(int),
 						  _variant_t("0")) );

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nError",
                          ADODB::adInteger, 
                          ADODB::adParamOutput, 0,_variant_t("0")));


		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter("@strPwdHash",
                          ADODB::adVarWChar, 
                          ADODB::adParamOutput, 255,_variant_t("")));

		_variant_t vMAC = (LPCTSTR)m_pDlg->GetMAC1();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strMAC",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vMAC));

		_variant_t vMAC2 = (LPCTSTR)m_pDlg->GetMAC2();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strMAC2",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vMAC2));
		
		_variant_t vASN = (LPCTSTR)m_pDlg->GetSNEnvelope();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strASN",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vASN));
		

		_variant_t vSN = (LPCTSTR)m_pDlg->GetSNPhone();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strSN",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vSN));
		
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nKeys",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)nKeys)));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nLeds",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)nLeds)));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nAudio",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)nAudio)));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nDisplay",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)nDisplay)));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nIsMAC1OK",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)((0 == m_pDlg->GetMAC1().CompareNoCase(m_strMAC)) ? 1 : 0))));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nIsMAC2OK",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)((0 == m_pDlg->GetMAC2().CompareNoCase(m_strMAC2)) ? 1 : 0))));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nIsSNOK",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)((0 == m_pDlg->GetSNPhone().CompareNoCase(m_strSN)) ? 1 : 0))));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nIsASNOK",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)((0 == m_pDlg->GetSNEnvelope().CompareNoCase(m_strASN)) ? 1 : 0))));

		ptrCommand->Execute(NULL,NULL,ADODB::adCmdStoredProc );

		
		_variant_t propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 1 ) ]->Value;
		nError = propValue.intVal;
		propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 2 ) ]->Value;
		m_strPwdHash = (LPCTSTR)(propValue.bstrVal);

		ptrConnection->Close();
	}
	catch(_com_error &e)
	{
	// Notify the user of errors if any.
    // Pass a connection pointer accessed from the Connection.
//		PrintProviderError(m_ptrConnection);
      _bstr_t bstrSource(e.Source());
      _bstr_t bstrDescription(e.Description());

//      PrintProviderError(pConnection);
		CString str;
		str.Format(IDS_ERR_DB,
			(LPCTSTR)bstrSource,(LPCTSTR)bstrDescription);
		AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
	  nError = 1;
	}
	return nError;
}


int CPhone::SetPasswordStatus(bool bIsPwdSet)
{
#ifdef NO_DB
	return 0;
#endif
	ADODB::_CommandPtr		ptrCommand;
	ADODB::_ConnectionPtr	ptrConnection;
	ADODB::_RecordsetPtr	ptrRecordSet;
	int nError ;

	HRESULT hr = S_OK;
	try
	{
		_bstr_t strCnn( (LPCTSTR)GetConnectionString() );
	    hr = ptrConnection.CreateInstance(__uuidof( ADODB:: Connection ));

		ptrConnection->Open(strCnn, "", "", ADODB::adOpenUnspecified);
	
 		hr = ptrCommand.CreateInstance(__uuidof( ADODB:: Command ));
		
	// Tell the Command to use our existing Connection.
		ptrCommand->ActiveConnection = ptrConnection;

		CString strProcedureName;
		strProcedureName = _T("sp_SetPasswordStatus");

		ptrCommand->CommandText = (LPCTSTR)strProcedureName;
		ptrCommand->CommandType = ADODB::adCmdStoredProc; 
//		SetConnectionEvents(ptrConnection);

		
	// Add stored procedure params
		ptrCommand->Parameters->Append(
			ptrCommand->CreateParameter( "@RETURN_VALUE",
                          ADODB::adInteger, 
                          ADODB::adParamReturnValue, 
                          sizeof(int),
 						  _variant_t("0")) );

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@nError",
                          ADODB::adInteger, 
                          ADODB::adParamOutput, 0,_variant_t("0")));


		_variant_t vMAC = (LPCTSTR)m_pDlg->GetMAC1();
		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@strMAC",
                      ADODB::adVarWChar, 
                      ADODB::adParamInput, 12, vMAC));

		ptrCommand->Parameters->Append(		
			ptrCommand->CreateParameter( "@bIsPwdSet",
                          ADODB::adInteger, 
                          ADODB::adParamInput, sizeof(int),_variant_t((long)bIsPwdSet)));
		
		ptrCommand->Execute(NULL,NULL,ADODB::adCmdStoredProc );

		
		_variant_t propValue = ptrCommand->Parameters->Item[ _variant_t( (long) 1 ) ]->Value;
		nError = propValue.intVal;

		ptrConnection->Close();
	}
	catch(_com_error &e)
	{
	// Notify the user of errors if any.
    // Pass a connection pointer accessed from the Connection.
//		PrintProviderError(m_ptrConnection);
      _bstr_t bstrSource(e.Source());
      _bstr_t bstrDescription(e.Description());

//      PrintProviderError(pConnection);

		CString str;
		str.Format(IDS_ERR_DB,
			(LPCTSTR)bstrSource,(LPCTSTR)bstrDescription);
		AfxMessageBox(str, MB_OK | MB_ICONSTOP); 
	    nError = 1;
	}
	return nError;
}


CString CPhone::GetConnectionString() const
{
	CString strConnectString;
	strConnectString.Format(TEST_CONN_STRING,
			AfxGetApp()->GetProfileString( _T("DB"), _T("Server"), _T("")), 
			AfxGetApp()->GetProfileString( _T("DB"), _T("Name"), _T("") ),
			AfxGetApp()->GetProfileString( _T("DB"), _T("User"), _T("") ),
			AfxGetApp()->GetProfileString( _T("DB"), _T("Password"), _T("") ));

	return strConnectString;
}	
