#ifndef _PHONE_H_
#define _PHONE_H_

#define PHONE_KEYS	30
#define PHONE_LEDS	2

#include "SocketPhone.h"

class CPhoneDlg;

class CPhone : public CSocketPhone
{
public:
	enum eTest
	{
		tKbd = 0,
		tLED1red,
		tLED1green,
		tLED1off,
		tLED2red,
		tLED2green,
		tLED2off,
		tDisplayFull,
		tDisplayMesh,
		tDisplayEmpty,
		tHandsetSpeaker,
		tHandsetMic,
		tPhoneSpeaker,
		tPhoneMic,
		tLast 
	};
private:
	struct Key
	{
		int nUp;
		int nDown;
	};
	struct Led
	{
		bool bOff;
		bool bRed;
		bool bGreen;
	};
	struct Audio
	{
		bool bHandsetSpeaker;
		bool bHandsetMic;
		bool bPhoneSpeaker;
		bool bPhoneMic;
	};
	struct Display
	{
		bool bFull;
		bool bMesh;
		bool bEmpty;
	};
public:
	CPhone();

public:
	bool IsTestCompleted() const { return m_nTest == tLast; }
	bool TestsOK() const;
	bool IsPhoneOK() const { return m_bPhoneOK; }

	const BYTE* GetMAC() const {return m_mac; }
	void SetDlg(CPhoneDlg *pDlg);
protected:
	bool StartTest();
	bool NextTest();

	bool TestKbd();
	bool TestLED();
	bool TestDisplay();

	bool TestAudio();
	void MarkButtonAsChecked(int nButton, bool bUp);
	void Init();

	int GetDBPhoneInfo();
	int GetDBPwdHash();

	int StoreTestResults();
	int SetPasswordStatus(bool bIsPwdSet);
	CString GetConnectionString() const;


public:
	bool OnConnected();
	bool OnData(LPCTSTR lpData);
	bool OnDisconnected();
	bool OnTestFinish(bool bOK);
	

private:
	eTest				m_nTest;		
	Key					m_keys[PHONE_KEYS];
	Led					m_leds[PHONE_LEDS];
	Audio				m_audio;
	Display				m_display;
	BYTE				m_mac[8];
	CPhoneDlg*			m_pDlg;

	bool				m_bPhoneOK;
	bool				m_bAlreadyTested;
public:
	CString				m_strMAC; 
	CString				m_strMAC2; 
	CString				m_strASN;
	CString				m_strSN;
	CString				m_strPwdHash;

};

#endif // _PHONE_H_