// PhonePicture.cpp : implementation file
//

#include "stdafx.h"
#include "PXATester.h"
#include "PhonePicture.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


RECT oButtons[PHONE_BUTTONS] =
{
	{177, 44, 232, 75}, 
	
	{290, 42, 320, 59}, 
	{284, 71, 320, 88}, 
	{290, 100, 320, 117}, 
	
	{481, 42, 512, 59}, 
	{481, 71, 518, 88}, 
	{481, 100, 512, 117}, 

	{359, 164, 374, 179}, 
	{392, 164, 409, 179}, 
	{427, 164, 443, 179}, 

	{300, 190, 343, 210}, 
	{300, 226, 343, 244}, 
	{300, 260, 343, 278}, 
	{300, 294, 343, 312}, 

	{354, 186, 381, 213}, 
	{354, 222, 381, 248}, 
	{354, 256, 381, 282}, 
	{354, 290, 381, 316}, 
	
	{388, 186, 415, 213}, 
	{388, 222, 415, 248}, 
	{388, 256, 415, 282}, 
	{388, 290, 415, 316}, 

	{423, 186, 450, 213}, 
	{423, 222, 450, 248}, 
	{423, 256, 450, 282}, 
	{423, 290, 450, 316}, 

	{462, 190, 505, 210}, 
	{462, 226, 505, 244}, 
	{462, 260, 505, 278}, 
	{462, 294, 505, 312}, 
};

RECT oLEDs[PHONE_LEDS] =
{
	{317, 175, 325, 183},  
	{477, 175, 485, 183}
};

RECT rectDisplay =
	{337,49, 465, 113};

RECT oDevices[PHONE_DEVICES] =
{
	{50, 43, 76, 69}, 
	{46, 277, 69, 292}, 
	{166, 176, 236, 241}, 
	{506, 140, 528, 152}
};



#define DISPLAY_BGCOLOR RGB(0x83, 0x9C, 0x6A)


void DrawTransparentBitmap(HDC hdc, HBITMAP hBitmap, short xStart,
                           short yStart, COLORREF cTransparentColor);

/////////////////////////////////////////////////////////////////////////////
// CPhonePicture

CPhonePicture::CPhonePicture()
{
	Init();
}

CPhonePicture::~CPhonePicture()
{
}

void CPhonePicture::Init()
{
	int i;
	for(i=0; i<PHONE_BUTTONS; i++)
	{
		m_oButtons[i]=false;
	}
	for(i=0; i<PHONE_LEDS; i++)
	{
		m_oLEDs[i]=0;
	}
	m_nDisplay = 2;
	m_nDevice = -1;
	m_nStatus = -1;
}

BEGIN_MESSAGE_MAP(CPhonePicture, CWnd)
	//{{AFX_MSG_MAP(CPhonePicture)
	ON_WM_DRAWITEM()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPhonePicture message handlers
void CPhonePicture::OnDraw(CDC& dc) 
{
	int i;
	for(i=0; i<PHONE_BUTTONS; i++)
	{
		if (!m_oButtons[i])
		{
			CRect rect = oButtons[i];
			dc.FillSolidRect(rect, RGB(255,0,0));
	
		}
	}
	for(i=0; i<PHONE_LEDS; i++)
	{
		int nColor = RGB((m_oLEDs[i] & 1 ? 255:0), (m_oLEDs[i]&2 ? 255:0),0);
		CBrush brush;
		brush.CreateSolidBrush(nColor);
		dc.SelectObject(brush);
		dc.Ellipse(&oLEDs[i]);
	}	
	DrawDisplay(dc);
	if (m_nDevice>=0)
	{
		CBrush brush;
		brush.CreateSolidBrush(RGB(255,0,0));
		dc.SelectObject(brush);
		CRect rect = oDevices[m_nDevice];
		dc.FillSolidRect(rect, RGB(255,0,0));
	}
	if (m_nStatus >=0)
	{
		LOGFONT lf;                        // Used to create the CFont.
		CFont f;
		memset(&lf, 0, sizeof(LOGFONT));   // Clear out structure.
		lf.lfHeight = 70;                  // Request a 20-pixel-high font
		_tcscpy(lf.lfFaceName, _T("Arial"));    //    with face name "Arial".
		lf.lfCharSet = EASTEUROPE_CHARSET;
		f.CreateFontIndirect(&lf);    // Create the font.
		dc.SelectObject(f);
		CRect rect;
		GetClientRect(rect);
		CString str;
		dc.SetBkMode(TRANSPARENT);
		dc.SetTextColor(m_nStatus ==0 ? RGB(0,255,0): RGB(255,0,0));
		str.LoadString(m_nStatus ==0 ? IDS_PHONE_OK: IDS_PHONE_WRONG);
		dc.DrawText(str, &rect, DT_VCENTER | DT_SINGLELINE | DT_CENTER);
	}
}

void CPhonePicture::DrawDisplay(CDC& dc) 
{
	dc.FillSolidRect(&rectDisplay, DISPLAY_BGCOLOR);
	CRect rect(0,0,128,64);
	rect.OffsetRect(
		rectDisplay.left+(rectDisplay.right-rectDisplay.left-rect.Width())/2, 
		rectDisplay.top+(rectDisplay.bottom-rectDisplay.top-rect.Height())/2);
	
	switch(m_nDisplay)
	{
		case 0: // Full
			dc.FillSolidRect(&rect, RGB(0,0,0));
		break;
		case 1:	// Grid
		{
			int i;
			for(i=rect.top; i<rect.bottom; i+=2)
				dc.FillSolidRect(rect.left, i, rect.Width(), 1, RGB(0,0,0));
			for(i=rect.left; i<rect.right; i+=2)
				dc.FillSolidRect(i, rect.top, 1, rect.Height(), RGB(0,0,0));
		}
		break;
	}
}

void CPhonePicture::SetButtonStatus(int nButton, bool bChecked)
{
	m_oButtons[nButton] = bChecked;
	RedrawWindow(&oButtons[nButton], NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CPhonePicture::SetLEDStatus(int nLED, bool bRed, bool bGreen)
{
	m_oLEDs[nLED] = (bRed ? 1:0) | (bGreen ? 2:0);
	RedrawWindow(&oLEDs[nLED], NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CPhonePicture::SetDisplayStatus(int nDisplay)
{
	m_nDisplay = nDisplay;
	RedrawWindow(&rectDisplay, NULL, RDW_INVALIDATE | RDW_UPDATENOW);

}

void CPhonePicture::ShowAudioDevice(int nDevice)
{
	int nOld = m_nDevice;
	m_nDevice = nDevice;
	if (-1 != nOld)
	{
		RedrawWindow(&oDevices[nOld], NULL, RDW_INVALIDATE | RDW_UPDATENOW);
	}
	RedrawWindow(&oDevices[nDevice], NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CPhonePicture::SetStatus(int nStatus)
{
	m_nStatus = nStatus;
	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CPhonePicture::OnPaint() 
{
	CRect rect;
	GetUpdateRect(rect);
	CStatic::OnPaint();
	CDC *pDC = GetDC();
	if (pDC)
	{
		pDC->SetBoundsRect(rect, DCB_DISABLE);
		OnDraw(*pDC);
		ReleaseDC(pDC);
	}
}	

