// SocketPhone.cpp : implementation file

#include "stdafx.h"
#include "PXATester.h"
#include "SocketPhone.h"
#include "Helpers.h"
#include <afxpriv.h>
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#define CONNECTION_TIMEOUT 3000
/////////////////////////////////////////////////////////////////////////////
// CSocketPhone

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CSocketPhone, CAsyncSocket)
	//{{AFX_MSG_MAP(CSocketPhone)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

CSocketPhone::CSocketPhone()
{
	m_nBytesSent = m_nSendDataLen = 0;
	m_nRecvDataLen = sizeof(int);	// initialize for 4 byte data length
	m_nBytesRecv = 0;
	m_bReadDataLength = TRUE;
	m_bConnected = false;
}

CSocketPhone::~CSocketPhone()
{
}


/////////////////////////////////////////////////////////////////////////////
// CSocketPhone member functions
void CSocketPhone::OnConnect(int nErrorCode) 
{
//	Start();
	OnConnected();
}

void CSocketPhone::OnClose(int nErrorCode) 
{
	m_bConnected = false;
	// passive close	
	ShutDown();		
	Close();

	OnDisconnected();
	CAsyncSocket::OnClose(nErrorCode);
}

// Receives data with a 4 byte preappended data length header.    
// The function is set up to call Receive only once as recommended.
void CSocketPhone::OnReceive(int nErrorCode) 
{
	int nRead = 0;
	m_dwLastRead = GetTickCount();
	// data needs to be read (which should be all the time when this is called)
	if (m_nBytesRecv < m_nRecvDataLen) 
	{
		// make sure we're reading 4 bytes for data length
		if (m_bReadDataLength)
			ASSERT(m_nRecvDataLen == sizeof(int));

		// receive buffer max size is MAX_BUFF
		// We must have enough room available in buffer AND
		// the expected packet size must be less or equal to MAX_BUFF
		ASSERT(m_nBytesRecv < MAX_BUFF && m_nRecvDataLen <= MAX_BUFF);

		// read all the data
		nRead = Receive(
						m_recvBuff + m_nBytesRecv,
						m_nRecvDataLen-m_nBytesRecv);

		// if something was read
		if (nRead > 0) 
		{
			m_nBytesRecv += nRead;

			// if all the data was read
			if (m_nBytesRecv == m_nRecvDataLen) 
			{
				// extract data length from packet
				if (m_bReadDataLength) 
				{
					m_nRecvDataLen = (int) ntohl( (*((int*)m_recvBuff)) );
					m_nBytesRecv = 0;
				} 
				else 
				{
					// extract data from buffer and pass data to the upper layer.
					// We append the body of the packet with a string terminator.

					if (m_nRecvDataLen <= MAX_BUFF)
						m_recvBuff[m_nRecvDataLen] = '\0';
					else
						m_recvBuff[MAX_BUFF] = '\0';

					OnData(MBCS2String(m_recvBuff));

					// echo data back to client
//					AsyncSendBuff(m_recvBuff, m_nRecvDataLen);
					
					// re-initializaton
					m_nRecvDataLen = sizeof(int);
					m_nBytesRecv = 0;

				}
				// alternate between reading 4 byte data length and the data itself
				m_bReadDataLength = m_bReadDataLength ? FALSE : TRUE;
			}
		} 
		else 
		{	// else error occurred
			if (GetLastError() != WSAEWOULDBLOCK) 
			{
				m_nBytesRecv = m_nRecvDataLen;
				AfxMessageBox(_TEXT("Socket Error. Unable to read data."));
			} 
			else
;//				TRACE(_TEXT("CSocketPhone: WARNING: WSAEWOULDBLOCK on a Receive in OnReceive\n"));
		}
	}
//	TRACE(_TEXT("CSocketPhone::OnReceive(int nErrorCode = %d) nRead = %d\n"), nErrorCode, nRead);
}


void CSocketPhone::OnSend(int nErrorCode) 
{
	// we called CAsyncSocket::Send and it returned WSAEWOULDBLOCK
	OutputDebugString(_TEXT("CSocketPhone::OnSend\n"));
	DoAsyncSendBuff();	
	CAsyncSocket::OnSend(nErrorCode);
}

void CSocketPhone::AsyncSendBuff(void* lpBuf, int nBufLen)
{
	// We don't queue up data here.
	// If you are going to queue up data packet, it would be better to limit the size
	// of the queue and remember to clear up the queue whenever the current packet has been sent.
	if (m_nSendDataLen != 0 || nBufLen > MAX_BUFF)
	{
		TCHAR szError[256];
		wsprintf(szError, _TEXT("CSocketPhone::AsyncSendBuff() can't accept more data\n"));
		AfxMessageBox (szError);
		return;
	}
	else
	{
		if (nBufLen > MAX_BUFF)
		{
			TCHAR szError[256];
			wsprintf(szError, _TEXT("CSocketPhone::AsyncSendBuff() oversize buffer.\n"));
			AfxMessageBox (szError);
			return;
		}
		int nHeaderLen = (int) htonl(nBufLen);
		// 4 byte header
		memcpy(m_sendBuff, &(nHeaderLen), sizeof(int));
		// body
		memcpy(m_sendBuff+ sizeof(int), lpBuf, nBufLen);
		m_nSendDataLen = nBufLen + sizeof(int);
		m_nBytesSent = 0;
		DoAsyncSendBuff();
	}
}

// Send the data left in the buffer.
// If TCP stack can not accept more data and gives error of WSAEWOULDBLOCK,
// we breaks out of the while loop. Whenever TCP stack can accept more data,
// our CSocketPhone::OnSend() will be called.
void CSocketPhone::DoAsyncSendBuff()
{
	while (m_nBytesSent < m_nSendDataLen)  
	{
		int nBytes;

		if ((nBytes = Send((LPCTSTR)m_sendBuff + m_nBytesSent, m_nSendDataLen - m_nBytesSent))
			== SOCKET_ERROR)
		{
			if (GetLastError() == WSAEWOULDBLOCK) 
				break;
			else
			{
				TCHAR szError[256];
				wsprintf(szError, _TEXT("Server Socket failed to send: %d"), GetLastError());
				
				Close();
			
				AfxMessageBox (szError);

				m_nBytesSent = 0;
				m_nSendDataLen = sizeof(int);
				return;
			}
		}
		else
		{
			m_nBytesSent += nBytes;
		}
	}

	if (m_nBytesSent == m_nSendDataLen)
	{
		m_nBytesSent = m_nSendDataLen = 0;
	}
}

int CSocketPhone::Receive(void* lpBuf, int nBufLen, int nFlags) 
{
	OutputDebugString(_TEXT("CSocketPhone::Receive\n"));
	return CAsyncSocket::Receive(lpBuf, nBufLen, nFlags);
}

int CSocketPhone::Send(const void* lpBuf, int nBufLen, int nFlags) 
{
	OutputDebugString(_TEXT("CSocketPhone::Send\n"));
	return CAsyncSocket::Send(lpBuf, nBufLen, nFlags);
}

bool CSocketPhone::Send(const CString& str)
{
	char* buff = String2MBCS(str);
	AsyncSendBuff(buff, strlen(buff));
	delete [] buff;
	return true;
}

UINT CSocketPhone::ConnectionTester( LPVOID pParam )
{
	CSocketPhone *pSocket = (CSocketPhone *)pParam;
	while(pSocket->m_bConnected)
	{
		Sleep(CONNECTION_TIMEOUT);
		if ((!pSocket->m_bSuspended) && ((pSocket->m_dwLastRead + CONNECTION_TIMEOUT) < GetTickCount()))
		{	// Broken Connection
			if (pSocket->m_bConnected)
				::PostMessage(pSocket->m_hSocketWindow, WM_SOCKET_NOTIFY, (WPARAM)pSocket->m_hSocket, FD_CLOSE);
			break;	
		}
	}
	return 0;
}


void CSocketPhone::StartChecker() 
{
	m_bSuspended = false;
	m_bConnected = true;
	m_dwLastRead = GetTickCount();

	AFX_MODULE_THREAD_STATE* pState = AfxGetModuleThreadState();

	m_hSocketWindow = pState->m_hSocketWindow;

	AfxBeginThread( CSocketPhone::ConnectionTester, this, THREAD_PRIORITY_NORMAL, 0, 0, NULL );
}

void CSocketPhone::SuspendChecker() 
{
	m_bSuspended = true;
}

void CSocketPhone::ResumeChecker() 
{
	m_dwLastRead = GetTickCount();
	m_bSuspended = false;
}